package Stanford::Shibboleth::IDP ;

use Carp ;
use Data::Dumper ;
use Stanford::Orange::Util qw/ run_command_improved trim / ;

# Get values from the idp.properties file
#
# Returns a hash with these keys:
#    idp.sealer.keyPassword
#    idp.sealer.storeResource   (the full path to the DataSealer JKS file)
#    idp.sealer.versionResource (the full path to the DataSealer version number file)

sub read_idp_properties {
    my ($config_file) = @_ ;

    my $DEFAULT_IDP_PROPERTIES_FILE = '/opt/shibboleth-idp/conf/dynamic.properties' ;

    if (!defined($config_file)) {
        $config_file = $DEFAULT_IDP_PROPERTIES_FILE ;
    }

    my $config = Config::IniFiles->new(-file     => $config_file,
                                       -fallback => 'default'
        ) ;

    my %keytovalue = () ;

    my $IDP_HOME = '/opt/shibboleth-idp' ;

    ## #####     ## #####     ## #####     ## #####     ## #####
    my $store_values_fref = sub {
        my ($keyname) = @_ ;
        my $value = $config->val('default', $keyname) ;
        if (!$value) {
            my $msg = "could not find value for '$keyname' in configuration file" ;
            croak $msg ;
        } else {
            # Replace '%{idp.home}' with $IDP_HOME
            $value =~ s/%{idp.home}/$IDP_HOME/g ;

            $keytovalue{$keyname} = $value ;
        }
    } ;
    ## #####     ## #####     ## #####     ## #####     ## #####

    my @keynames = qw(
        idp.sealer.keyPassword
        idp.sealer.storeResource
        idp.sealer.versionResource
        ) ;

    foreach my $keyname (@keynames) {
        $store_values_fref->($keyname) ;
    }

    return %keytovalue ;
}

# Set the JAVA_HOME environment variable if not already set.
sub set_java_home {
    if (!$ENV{'JAVA_HOME'}) {
        $ENV{'JAVA_HOME'} = '/usr/lib/jvm/java-8-openjdk-amd64/jre/' ;
    }
}


# Add a new sealer password to the Java keystore sealer file.
# Note that we add a maximum of $COUNT passwords; when we reach this
# maximum we remove passwords on a FIFO (first-in first-out basis).

# $count: maximum number of passwords to save (defaults to 60).
sub add_new_sealer_password {
    my (%args) = (
        'count'           => 60,
        'alias'           => 'secret',
        'idp-home'        => '/opt/shibboleth-idp',
        'properties_file' => undef,
        @_,
        ) ;

    my $count           = $args{'count'} ;
    my $alias           = $args{'alias'} ;
    my $idp_home        = $args{'idp-home'} ;
    my $properties_file = $args{'properties_file'} ;

    my %parameters = read_idp_properties($properties_file) ;

    my $keystore_file = $parameters{'idp.sealer.storeResource'} ;
    my $version_file  = $parameters{'idp.sealer.versionResource'} ;

    # The $keyPassword argument is used to access the passwords stored
    # in the keystore file.
    my $keyPassword = $parameters{'idp.sealer.keyPassword'} ;

    set_java_home() ;

    my @cmd = ("$idp_home/bin/seckeygen.sh",
               '--storefile',   $keystore_file,
               '--storepass',   $keyPassword,
               '--count',       $count,
               '--versionfile', $version_file,
               '--alias',       $alias,
        ) ;

    my ($stdout, $stderr, $rc) = run_command_improved(@cmd) ;

    if ($stderr) {
        croak "seckeygen failed: $stderr" ;
    }

    # Make sure the permissions and owner are correct.
    my $uid = getpwnam 'tomcat8' ;
    my $gid = getgrnam 'tomcat8' ;
    chown $uid, $gid, $keystore_file ;
    chown $uid, $gid, $version_file ;
    chmod 0640, $keystore_file ;
    chmod 0640, $version_file ;

    return 1 ;
}

sub show_sealer_password {
    my ($properties_file) = @_ ;

    my %parameters = read_idp_properties($properties_file) ;

    set_java_home() ;

    my $keystore_file = $parameters{'idp.sealer.storeResource'} ;
    my $keyPassword   = $parameters{'idp.sealer.keyPassword'} ;

    my @cmd = (
        'keytool',
        '-list',
        '-keystore',  $keystore_file,
        '-storetype', 'JCEKS',
        '-storepass', $keyPassword,
        ) ;

    my ($stdout, $stderr, $rc) = run_command_improved(@cmd) ;

    if ($stderr) {
        croak "keytool list failed: $stderr" ;
    } else {
        return trim($stdout) . "\n" ;
    }
}



1;

__END__

=head1 NAME

Stanford::Shibboleth::IDP - Stanford IdP Perl Library

=head1 VERSION

Version 0.01

=head1 ABSTRACT

The Stanford::Shibboleth::IDP class ...

=head1 AUTHOR

Adam Lewenberg, C<< <adamhl at stanford.edu> >>

=head1 BUGS

Please report any bugs or feature requests
to C<adamhl at stanford.edu>.

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc Stanford::Shibboleth::IDP

=head1 COPYRIGHT & LICENSE

Copyright 2016 Stanford University, all rights reserved.

=cut
